/***************************************************************************
  $RCSfile: test.cpp,v $
  -------------------
  cvs         : $Id: test.cpp,v 1.2 2006/01/21 11:47:03 aquamaniac Exp $
  begin       : Wed Oct 15 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

/* Internationalization */
#ifdef ENABLE_NLS
# include <libintl.h>
# include <locale.h>
# define I18N(m) gettext(m)
#else
# define I18N(m) m
#endif



#include "adminjobs.h"
#include "test.h"
#include "accountjobs.h"

#include <gwenhywfar/debug.h>
#include <gwenhywfar/text.h>


ModuleTest::ModuleTest()
:AqmoneyModule() {
}



ModuleTest::~ModuleTest(){
}



void ModuleTest::dumpDescription(FILE *f){
  fprintf(f, "%s",
          I18N("Updates bank and account information\n"
               "    This is most likely needed after updating your XML"
               "files\n"));
}


void ModuleTest::_usage(){
  fprintf(stderr, "\n");
  fprintf(stderr, "%s aqmoney2 test [...]\n%s\n",
	  I18N("Usage:"),
	  I18N("This is an internal module for testing purposes\n"));
  fprintf(stderr, "\n");
  dumpDescription(stderr);
  fprintf(stderr, "\n%s\n\n", I18N("See aqmoney2(1) for detailed help."));
}



int ModuleTest::_checkArgs(s_args &args, int argc, char **argv){
  int i;
  string tmp;

  i=0;
  args.country=0;
  args.bankCode="*";
  args.customerId="*";

  while (i<argc){
    tmp=argv[i];
    if (tmp.empty()) {
      fprintf(stderr, I18N("Error: Empty command specific argument\n"));
      return -1;
    }
    if (tmp=="-c") {
      i++;
      if (i>=argc)
	return -1;
      args.country=atoi(argv[i]);
    }
    else if (tmp=="-b") {
      i++;
      if (i>=argc)
        return -1;
      args.bankCode=argv[i];
    }
    else if (tmp=="-C") {
      i++;
      if (i>=argc)
        return -1;
      args.customerId=argv[i];
    }
    else if (tmp=="-h" || tmp=="--help") {
      _usage();
      return -2;
    }
    else {
      if (tmp[0]=='-') {
        fprintf(stderr, I18N("Option \"%s\" is unknown to this command\n"),
                tmp.c_str());
        return -1;
      }
      args.params.push_back(argv[i]);
    }
    i++;
  } // while
  // that's it
  return 0;
}




ModuleError ModuleTest::run(AqMoneyAPI *api,
                            int argc, char **argv,
                            const string &openhbciFile,
                            const string &aqmoneyFile){
  int rv;
  Error err;
  Pointer<Outbox> queue;
  Pointer<Customer> cust;
  Pointer<User> user;
  Pointer<Bank> bank;
  list<Pointer<Customer> > custs;
  list<Pointer<Customer> >::iterator cit;
  Pointer<Medium> medium;
  bool errorFlag;
  unsigned int seq;

  rv=_checkArgs(_args,argc,argv);
  if (rv==-2)
    return ModuleErrorSuccess;
  else if (rv==-1) {
    _usage();
    return ModuleErrorArguments;
  }

  err=api->loadEnvironment(openhbciFile, aqmoneyFile);
  if (!err.isOk()) {
    DBG_ERROR(0, "Error loading environment (%s)",
              err.errorString().c_str());
    return ModuleErrorLoad;
  }

  errorFlag=0;
  custs=api->getCustomers(_args.country,
                          _args.bankCode,
                          _args.customerId);
  if (custs.empty()) {
    fprintf(stderr,
            "%s",
            I18N("Error: No matching customers found.\n"));
    return ModuleErrorInvalidArgs;
  }
  if (custs.size()>1) {
    fprintf(stderr,
            "%s",
            I18N("Error: Ambiguous customer specification.\n"));
    return ModuleErrorInvalidArgs;
  }
  cust=custs.front();
  user=cust.ref().user();
  medium=user.ref().medium();
  bank=user.ref().bank();

  try {
    string testData;
    string signature;
    string encrypted;
    string key;
    string enckey;

    // mount medium
    err=medium.ref().mountMedium();
    if (!err.isOk()) {
      fprintf(stderr,
	      I18N("Error mounting medium (%s)\n"),
	      err.errorString().c_str());
      return ModuleErrorExecution;
    }

    testData=medium.ref().mediumId();
    fprintf(stderr, "MediumId is:\n");
    GWEN_Text_DumpString(testData.data(), testData.length(),
                         stderr, 2);


    err=medium.ref().selectContext(bank.ref().country(),
				   bank.ref().bankCode(),
				   user.ref().userId());
    if (!err.isOk()) {
      medium.ref().unmountMedium();
      fprintf(stderr,
	      I18N("Error selecting user context (%s)\n"),
	      err.errorString().c_str());
      return ModuleErrorExecution;
    }

    fprintf(stderr, "Reading next seq...\n");
    seq=medium.ref().nextSEQ();
    fprintf(stderr, "Next Seq is: %d\n", seq);

    testData="This is a little test...";
    signature=medium.ref().sign(testData);
    fprintf(stderr, "Signature is:\n");
    GWEN_Text_DumpString(signature.data(), signature.length(),
                         stderr, 2);

    key=medium.ref().createMessageKey();
    fprintf(stderr, "Key is:\n");
    GWEN_Text_DumpString(key.data(), key.length(),
			 stderr, 2);

    enckey=medium.ref().encryptKey(key);
    fprintf(stderr, "Encrypted key is:\n");
    GWEN_Text_DumpString(enckey.data(), enckey.length(),
			 stderr, 2);

    encrypted=medium.ref().encrypt(testData, key);
    fprintf(stderr, "Encrypted data is:\n");
    GWEN_Text_DumpString(encrypted.data(), encrypted.length(),
			 stderr, 2);

    err=medium.ref().unmountMedium();
    if (!err.isOk()) {
      fprintf(stderr,
	      I18N("Error mounting medium (%s)\n"),
	      err.errorString().c_str());
      return ModuleErrorExecution;
    }
  } // try
  catch (Error xerr) {
    DBG_ERROR(0, "Error: %s", xerr.errorString().c_str());
    return ModuleErrorExecution;
  }

  if (errorFlag)
    return ModuleErrorExecution;
  return ModuleErrorSuccess;
}






